/****************************************************************************
** libebml : parse EBML files, see http://embl.sourceforge.net/
**
** <file/class description>
**
** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
**
** This file is part of libmatroska.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding an other license may use this file in accordance with 
** the Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.matroska.org/license/qpl/ for QPL licensing information.
** See http://www.matroska.org/license/gpl/ for GPL licensing information.
**
** Contact license@matroska.org if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

/*!
	\file
	\version \$Id: EbmlUnicodeString.h,v 1.7 2003/04/28 21:14:19 robux4 Exp $
	\author Steve Lhomme     <robux4 @ users.sf.net>
	\author Moritz Bunkus <moritz @ bunkus.org>
*/
#ifndef LIBEBML_UNICODE_STRING_H
#define LIBEBML_UNICODE_STRING_H

#include <string>

#include "EbmlConfig.h"
#include "EbmlTypes.h"
#include "EbmlElement.h"

#if defined(NO_WSTRING)
typedef std::string UTFstring;
#elif defined(LIBEBML_GCC2)// gcc 2.95 does not provide wstring
typedef basic_string<wchar_t> UTFstring;
#else
typedef std::wstring UTFstring;
#endif

START_LIBEBML_NAMESPACE

/*!
    \class EbmlUnicodeString
    \brief Handle all operations on a Unicode string EBML element
		\note internally treated as a string made of wide characters (16 bits ?)
*/
class EbmlUnicodeString : public EbmlElement {
	public:
		EbmlUnicodeString();
		EbmlUnicodeString(const UTFstring & DefaultValue);
		EbmlUnicodeString(const EbmlElement & ElementToClone);
		EbmlUnicodeString(const EbmlElement & ElementToClone, const UTFstring & aDefaultValue);
	
		virtual ~EbmlUnicodeString() {}
	
		bool ValidateSize() const {return true;} // any size is possible
		uint32 RenderData(IOCallback & output, bool bSaveDefault = false);
		uint64 ReadData(IOCallback & input);
		uint64 UpdateSize(bool bSaveDefault = false);
	
		EbmlUnicodeString & operator=(const UTFstring); ///< platform dependant code
#if !defined(NO_WSTRING)
		EbmlUnicodeString & operator=(const std::string); ///< UTF-8 to internal/platform
#endif // NO_WSTRING
// should not exist		operator const std::string &() const;
		operator const UTFstring &() const {return Value;}
	
		UTFstring DefaultVal() const {assert(DefaultIsSet); return DefaultValue;}

		bool IsDefaultValue() const {
			return (DefaultISset() && Value == DefaultValue);
		}

	protected:
		UTFstring Value; /// The actual value of the element
		uint32 DefaultSize; /// Minimum data size to fill on rendering
		UTFstring DefaultValue;
};

END_LIBEBML_NAMESPACE

#endif // LIBEBML_UNICODE_STRING_H
